@@ -8,7 +8,11 @@ Collection of common sphinx-needs functions for dynamic values
 
 import re
 
-from sphinxcontrib.needs.filter_common import filter_single_need, NeedInvalidFilter, filter_needs
+from sphinxcontrib.needs.filter_common import (
+    NeedInvalidFilter,
+    filter_needs,
+    filter_single_need,
+)
 from sphinxcontrib.needs.utils import logger
 
 
@@ -30,7 +34,7 @@ def test(app, need, needs, *args, **kwargs):
 
     :return: single test string
     """
-    return "Test output of need {}. args: {}. kwargs: {}".format(need['id'], args, kwargs)
+    return "Test output of need {}. args: {}. kwargs: {}".format(need["id"], args, kwargs)
 
 
 def copy(app, need, needs, option, need_id=None, lower=False, upper=False, filter=None):
@@ -115,10 +119,10 @@ def copy(app, need, needs, option, need_id=None, lower=False, upper=False, filte
     :param filter: :ref:`filter_string`, which first result is used as copy source.
     :return: string of copied need option
     """
-    if need_id is not None:
+    if need_id:
         need = needs[need_id]
 
-    if filter is not None:
+    if filter:
         result = filter_needs(needs.values(), filter, need)
         if result:
             need = result[0]
@@ -268,7 +272,7 @@ def check_linked_values(app, need, needs, result, search_option, search_value, f
         search_value = [search_value]
 
     for link in links:
-        if filter_string is not None:
+        if filter_string:
             try:
                 if not filter_single_need(needs[link], filter_string):
                     continue
@@ -363,24 +367,24 @@ def calc_sum(app, need, needs, option, filter=None, links_only=False):
 
     :return: A float number
     """
-    if not links_only:
-        check_needs = needs.values()
-    else:
+    if links_only:
         check_needs = []
         for link in need["links"]:
             check_needs.append(needs[link])
+    else:
+        check_needs = needs.values()
 
     calculated_sum = 0
 
     for check_need in check_needs:
-        if filter is not None:
+        if filter:
             try:
                 if not filter_single_need(check_need, filter):
                     continue
-            except ValueError as e:
+            except ValueError:
                 pass
             except NeedInvalidFilter as ex:
-                logger.warning('Given filter is not valid. Error: {}'.format(ex))
+                logger.warning("Given filter is not valid. Error: {}".format(ex))
         try:
             calculated_sum += float(check_need[option])
         except ValueError:
@@ -441,12 +445,12 @@ def links_from_content(app, need, needs, need_id=None, filter=None):
     :param filter: :ref:`filter_string`, which a found need-link must pass.
     :return: List of linked need-ids in content
     """
-    if need_id is not None:
+    if need_id:
         source_need = needs[need_id]
     else:
         source_need = need
 
-    links = re.findall(r':need:`(\w+)`|:need:`.+\<(.+)\>`', source_need['content'])
+    links = re.findall(r":need:`(\w+)`|:need:`.+\<(.+)\>`", source_need["content"])
     raw_links = []
     for link in links:
         if link[0] and link[0] not in raw_links:
@@ -454,7 +458,7 @@ def links_from_content(app, need, needs, need_id=None, filter=None):
         elif link[1] and link[0] not in raw_links:
             raw_links.append(link[1])
 
-    if filter is not None and len(filter) > 0:
+    if filter:
         filtered_links = []
         for link in raw_links:
             if link not in filtered_links and filter_single_need(needs[link], filter):
@@ -462,9 +466,3 @@ def links_from_content(app, need, needs, need_id=None, filter=None):
         return filtered_links
 
     return raw_links
-
-
-
-
-
-
